from lxml import etree
from .tags import TEITag
import sys

class _Parser():

    """Parser and configuration options"""

    def __init__(self):
        lookup = etree.ElementNamespaceClassLookup()
        self.parser = etree.XMLParser(**self.parser_options())
        self.parser.set_element_class_lookup(lookup)
        namespace = lookup.get_namespace('http://www.tei-c.org/ns/1.0')
        # We set TEITag as the default class for the parser.
        self._target_mapping(namespace)

    def _target_mapping(self, namespace):
        namespace[None] = self.base_tag_class()
        # Map the target to handling class in the lxml parser.
        for cls in self.tag_classes():
            if isinstance(cls.target, list):
                for target in cls.target:
                    namespace[target] = cls
            else:
                namespace[cls.target] = cls

    def __call__(self, textpath):
        """Parse textpath and return it"""
        return self._parse(str(textpath))

    def _parse(self, textpath):
        try:
            tree = etree.parse(textpath, self.parser)
        except FileNotFoundError:
            print('Could not find %s.' % textpath)
            sys.exit(1)
        except etree.XMLSyntaxError as err:
            print('An error occurred parsing {textpath}:\n\'{err}\''
                  .format(textpath=textpath, err=err))
            sys.exit(1)
        return tree

    def parser_options(self):
        """Options for lxml parser"""
        return {'ns_clean': True,
                'remove_comments': True,
                'remove_blank_text': True}

    def tag_classes(self):
        """Return classes to handle tags.
        You may well want to override this to include your own..."""
        return self._tag_classes()

    def base_tag_class(self):
        """Return the default class to handle tags"""
        return TEITag

    def _tag_classes(self, cls=TEITag):
        for subcls in cls.__subclasses__():
            yield from self._tag_classes(cls=subcls)
            if hasattr(subcls, 'target') and subcls.target:
                yield subcls


Parser = _Parser()

def transform_tree(tree, persdict, in_body=True):
    """Transform a tree"""
    # Using a heap here would be slower
    tags = list(tree.getiterator('*'))
    tags.sort()
    for tag in tags:
        if tag.localname == 'persName':
            if not in_body:
                tag.process(persdict, in_body=False)
            else:
                tag.process(persdict)
        else:
            tag.process()
    return tree



import subprocess
import re
from functools import partial

from .parser import Parser, transform_tree
from .pathsmanager import pathmanager
from .config import config
from .persdict import persdict


def transform_tei(inputname, outname=None,
                  force=False, standalone=False):
    """"Read inputname and transform to a pdf"""
    inputpaths, textwraps, workfiles = pathmanager(inputname, outname, standalone)
    bare_text = transform(*inputpaths)
    latex = latexify(bare_text, *textwraps)
    make_pdf(latex, *workfiles, force)


def transform(inputpath, personlistpath):
    _persdict = persdict(personlistpath)
    body = Parser(inputpath).getroot().find('.//{*}body')
    assert body is not None
    tree = transform_tree(body, _persdict)
    return '\n'.join(tree.itertext()).strip()


def latexify(bare_text, before, after):
    text = '\n'.join([before, bare_text, after])
    for fix in config['string_replacements']:
        text = text.replace(*fix)
    for fix in config['regex_replacements']:
        text = re.sub(*fix, text)
    return text


def make_pdf(latex, working_tex, working_pdf, out_pdf, force):

    missing = not working_pdf.exists() or not working_tex.exists()
    if force or missing or hash(working_tex.text()) != hash(latex):
        working_tex.write_text(latex)
        call_cmd = config['caller_command']
        latexmk = '{c} {w}'.format(c=call_cmd, w=working_tex).split()
        subprocess.call(latexmk)
    working_pdf.copy(out_pdf)



import argparse
from tei_transformer.transform import transform_tei
import sys


def get_args(args):
    parser = argparse.ArgumentParser(
        description="Transform a TEI-encoded critical edition into a pdf.")
    parser.add_argument("-t", "--transform",
                        help="TEI file to transform")
    parser.add_argument("-o", "--outputname",
                        help="Filename of the transformed file.",
                        default=None)
    parser.add_argument("-f", "--force",
                        help="Force recompilation of a latex file,\
                         even if unchanged.",
                        action="store_true")
    parser.add_argument('-q', '--quiet',
                        help="Run quietly",
                        action="store_true")
    parser.add_argument('-s', '--standalone',
                        help="Do not include introduction or appendices",
                        action="store_true")
    return parser.parse_args(args)


def interpret_args(args):
    if args.transform:
        transform_tei(
            args.transform,
            outname=args.outputname,
            force=args.force,
            standalone=args.standalone
            )


def main():
    args = get_args(sys.argv[1:])
    interpret_args(args)

if __name__ == '__main__':
    main()



from functools import partial
from .parser import Parser, transform_tree

def persdict(path):
    people = map(Person, Parser(path).getroot().iter('{*}person'))
    persdict = {person.xml_id: person for person in people}
    for person in persdict.values():
        person.update(persdict)
    return persdict


class Person():

    def __init__(self, tag):
        self.indexname, descript_name = self._names(tag)
        self.xml_id = tag.get('{http://www.w3.org/XML/1998/namespace}id')
        self.indexonly = self._indexonly(tag)
        self.description = self._description(tag, descript_name)

    def update(self, persdict):
        description, trait = self.description
        if trait is not None:
            transform_tree(trait, persdict, in_body=False)
        self.description = description(self._stripstring(trait))

    @staticmethod
    def _indexonly(tag):
        iattr = tag.get('indexonly') in ['true', 'True']
        itag = tag.find('{*}indexonly') is not None
        return iattr or itag

    @classmethod
    def _names(cls, tag):
        finder = partial(cls._find_string, tag.find('{*}persName'))
        targets = ['forename', 'addName', 'surname']
        firstnames, addnames, surnames = map(finder, targets)
        addnames = "`%s'" % addnames if addnames else ''
        all_names = [' '.join([firstnames, addnames]), surnames]
        return ', '.join(reversed(all_names)).strip(), ' '.join(all_names).strip()

    @classmethod
    def _dates(cls, tag):
        date_finder = partial(cls._find_string, tag)
        date_targets = ['birth', 'death']
        return map(date_finder, date_targets)

    @classmethod
    def _description(cls, tag, name):
        descript_fmt = '{} ({}--{}) {}'.format
        descript_part = partial(descript_fmt, name, *cls._dates(tag))
        trait_tag = tag.find('{*}trait')
        return descript_part, trait_tag

    @staticmethod
    def _stripstring(tag):
        try:
            return str(tag.text).strip()
        except AttributeError:
            return ''

    @classmethod
    def _find_string(cls, parent, query):
        """Return a string concatenating the text
        in each tag searchterm matches."""
        matches = (cls._stripstring(m) for m in parent.iter('{*}' + query))
        return ' '.join(matches)


import textwrap
import os
from functools import partial
from itertools import chain

from path import Path

from .config import config, update_config

class PathManager():

    def __init__(self, inputpath, outputname=None, standalone=False):
        self.inputpath = Path(inputpath)
        self.basename = self.inputpath.namebase
        self.outputname = outputname or self.basename + '.pdf'
        self.standalone = standalone
        self._dirs()
        self.latex_only_requirements()

    def _process(self, rnamelist):
        return map(self.process_resource, [config['resources'][x] for x in rnamelist])

    def latex_only_requirements(self):
        return self._process(['references', 'indexstyle'])

    def parsing_paths(self):
        return self.inputpath, next(self._process(['personlist']))

    def textwraps(self):
        before_text = ['preamble', 'after_preamble', 'after_text_start']
        after_text = ['after_text_end']
        return map('\n'.join, map(self._process, [before_text, after_text]))

    def latexmk_paths(self):
        working_tex, working_pdf = map(self.extend_workdir_b, ['.tex', '.pdf'])
        return working_tex, working_pdf, self.outputname

    def _dirs(self):
        curdir = self.inputpath.dirname() or os.curdir
        update_config(curdir)
        curdir = Path(curdir)
        resource_dir = self._extend_dir(curdir, 'resources', required=True)
        work_dir = self._extend_dir(curdir, config['workdir'], make=True) 
        self.extend_resourcedir = partial(self._extend_dir, resource_dir)
        self.extend_workdir = partial(self._extend_dir, work_dir)
        self.extend_workdir_b = lambda ext: self.extend_workdir(self.basename + ext)

    @staticmethod
    def _extend_dir(original, extension, make=False, required=False):
        path = Path(original.joinpath(extension))
        if not path.exists():
            if make:
                path.mkdir()
            elif required:
                raise IOError(path)
        return path

    def process_resource(self, resource):
        name, required, subst = map(resource.get, ['name', 'required', 'subst'])
        name = name or self.basename + resource['ext']
        r = self.extend_resourcedir(name)
        w = self.extend_workdir(name)
        if not r.exists() and required:
            raise FileNotFoundError(r)
        try:
            text = r.text()
        except FileNotFoundError as e:
            if subst in [None, False]:
                raise e
            text = self.process_subst(resource, subst)
        if resource.get('output') == 'read':
            return text
        w.write_text(text)
        return w

    def process_subst(self, resource, subst):
        sub_include = resource.get('sub_include')
        if sub_include and not self.standalone:
            _sub = {'name': sub_include['name'], 'subst': sub_include.get('subst') or ''}
            sub_include = self.process_resource(_sub)
            i = sub_include.namebase if sub_include else '%'
            include = '\\include{%s}' % i
            return subst % include
        return subst

def pathmanager(*args, **kwargs):
    manager = PathManager(*args, **kwargs)
    return manager.parsing_paths(), manager.textwraps(), manager.latexmk_paths()



from setuptools import setup, find_packages

setup(
    # Application name:
    name="tei_transformer",

    # Version number (initial):
    version="0.2.5",

    # Application author details:
    author="Tom McLean",
    author_email="thomasowenmclean@gmail.com",

    # Packages
    packages=["tei_transformer"],

    # Include additional files into the package
    include_package_data=True,
    package_data = {'tei_transformer': ['config.yaml']},

    entry_points={
    	'console_scripts': [
    	'tei_transformer=tei_transformer.tei_transformer_script:main',
    	]
    },

    # Details
    url="https://github.com/thomasowenmclean/tei_transformer",

    #
    license="GPL",
    description="Transform a TEI-encoded critical edition into a print-ready file.",

    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
		'Topic :: Text Processing :: Markup :: LaTeX',
        'Topic :: Text Processing :: Markup :: XML',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
    ],

    keywords='TEI critical edition xml transform scholarly diplomatic latex xml',

    # long_description=open("README.txt").read(),
    # Dependent packages (distributions)
    install_requires=[
        "lxml",
        "latexfixer",
        "path.py",
        "PyYAML"
    ],
)


from lxml import etree
import calendar

from latexfixer.fix import LatexText

LANGUAGES = {
    'it': 'italian',
    'de': 'german',
    'la': 'latin',
    'fr': 'french',
    'gr': 'greek',
}

XML_NAMESPACE = '{http://www.w3.org/XML/1998/namespace}'


class ImplementationError(Exception):
    pass


class FmtMethods():

    def _wrap(self, before, after, text=None):
        if text is None:
            text = self.text
        return '%s%s%s' % (before, text, after)

    def square_brackets(self, text=None):
        return self._wrap('[', ']', text)

    def single_quotes(self, text=None):
        return self._wrap('`', "'", text)

    def double_quotes(self, text=None):
        return self._wrap("``", "''", text)

    def emph(self, text=None):
        if text is None:
            text = self.text
        return '\\emph{%s}' % text

    def superscript(self, text=None):
        if text is None:
            text = self.text
        return '\\textsuperscript{%s}' % text

    def smallcaps(self, text=None):
        if text is None:
            text = self.text
        return '\\textsc{%s}' % text

    def handle_none(self):
        return self.text

    def _singles(self):
        return ['single']

    def _doubles(self):
        return ['double']

    def _emphs(self):
        return ['emph', 'italic', 'underscore', 'underline']

    def _supers(self):
        return ['super', 'upper']

    def _smcps(self):
        return ['smcp']

    def _none(self):
        return [None, 'none', 'None']

    def fmt_lists(self):
        return [(self._emphs(), self.emph),
                (self._singles(), self.single_quotes),
                (self._doubles(), self.double_quotes),
                (self._supers(), self.superscript),
                (self._smcps(), self.smallcaps),
                (self._none(), self.handle_none)]

class TEITag(etree.ElementBase):

    def _init(self):
        self._localname = None
        self.transform_text()
        self._descendants_count = None

    def transform_text(self):
        if self.text:
            self.text = self._process_text_contents(self.text)
        if self.tail:
            self.tail = self._process_text_contents(self.tail)

    def __eq__(self, other):
        return self.descendants_count == other.descendants_count

    def __lt__(self, other):
        return self.descendants_count < other.descendants_count

    def __str__(self):
        return etree.tounicode(self, with_tail=False)

    @property
    def descendants_count(self):
        if not self._descendants_count:
            if self.no_children():
                self._descendants_count = 0
            else:
                self._descendants_count = sum(self._descs())
        return self._descendants_count

    def _descs(self):
        for __ in self.iterdescendants():
            yield 1

    @property
    def localname(self):
        """Tag name without namespace"""
        if not self._localname:
            self._localname = etree.QName(self).localname
        return self._localname

    def process(self, persdict=None, in_body=True):

        # Check to see we won't accidentally wipe something out.
        not_good_children = self.localname not in ['choice', 'app']
        if not self.no_children() and not_good_children:
            self.raise_()
        # Now on with the replacement.
        if self.localname == 'persName':
            replacement = self.get_replacement(persdict, in_body)
        else:
            replacement = self.get_replacement()
        # We don't want to catch an empty string,
        if replacement is None:  # So we need the more specific test.
            return  # i.e. don't touch this.
        else:
            self.replace_w_str(replacement)

    def no_children(self):
        """Return true if self has no children"""
        return len(self) == 0

    def _process_text_contents(self, text):
        """Function called to manipulate text in tags or tail when first parsed.
        """
        return LatexText(text)

    def delete(self):
        """Remove this tag from the tree, preserving its tail"""
        parent = self.add_to_previous(self.tail)
        parent.remove(self)

    def get_replacement(self):
        """Called to get a string replacement for a tag"""
        raise NotImplementedError(self)

    def unwrap(self):
        """Replace tag with contents, including children"""
        children = list(self.iterchildren(reversed=True))
        if not len(children):
            self.replace_w_str(self.text)
        else:
            parent = self.getparent()
            index = parent.index(self)
            last_child = children[-1]
            last_child.tail = self.textjoin(last_child.tail, self.tail)
            parent = self.add_to_previous(self.textjoin(self.text, self.tail))
            for child in children:
                parent.insert(index, child)

    def replace_w_str(self, replacement):
        """Replace tag with string"""
        replacement = self.textjoin(replacement, self.tail)
        parent = self.add_to_previous(replacement)
        parent.remove(self)

    def add_to_previous(self, addition):
        """Add text to the previous tag"""
        previous = self.getprevious()
        parent = self.getparent()
        if previous is not None:
            previous.tail = self.textjoin(previous.tail, addition)
        else:
            parent.text = self.textjoin(parent.text, addition)
        return parent

    def raise_(self):
        """Raise an implementation error with self as argument"""
        raise ImplementationError(self)

    def required_key(self, key):
        x = self.get(key)
        if x:
            return x
        self.raise_()

    def _check_if_in_body(self):
        note_ancestors = self.iterancestors('{*}note')
        if any(True for _ in note_ancestors):
            return False
        return True

    @staticmethod
    def textjoin(a, b):
        """Join a and b, replacing either with an empty string
        if their value is not True"""
        return ''.join([(a or ''), (b or '')])

# STANDARD


class Head(TEITag):

    target = 'head'

    def get_replacement(self):
        """ Return the replacement for a tag of type <head>"""

        # Make explicit the desire not to indent following paragraph.
        if not self.tail or not self.tail.strip():
            next_sibling = self.getnext()
            if next_sibling is not None and next_sibling.localname == 'p':
                if next_sibling.get('rend') is None:
                    t = next_sibling.text or ''
                    next_sibling.text = '\\noindent %s' % t

        parent_attrs = self.getparent().attrib
        div_type = parent_attrs['type']
        if div_type == 'title':  # Chapter
            return self.process_head_level_two()

        try:
            identifier = parent_attrs['%sid' % XML_NAMESPACE]
        except KeyError:
            self.raise_()

        if div_type == 'diaryentry':  # Section
            return self.process_head_level_three(identifier)
        elif div_type == 'diaryentrysection':  # Subsection
            return self.process_head_level_four(identifier)
        self.raise_()

    def process_head_level_one(self):
        return '\\part{%s}' % self.text

    def process_head_level_two(self):
        return '\\pstart\n\\eledchapter*{%s}\n\\pend' % self.text

    def process_head_level_three(self, identifier):
        if identifier is None:
            self.raise_()
        month = identifier[:3]
        # Assumes gregorian calendar
        for index, string in enumerate(calendar.month_abbr):
            if string == month:
                month = calendar.month_name[index]
                break
        date = identifier[3:identifier.find('_')]
        year = identifier[-4:]
        return '\n\\pstart\n\
               \\eledsection[%s %s %s]{%s}\n\
               \\label{%s}\n\
               \\pend'\
               % (date, month, year, self.text, identifier)

    def process_head_level_four(self, identifier):
        return '\\pstart\n\\eledsubsection{%s}\n\\label{%s}\n\\pend'\
            % (self.text, identifier)

    def process_head_level_five(self, identifier):
        return '\\pstart\n\\eledsubsubsection{%s}\n\\label{%s}\n\\pend'\
            % (self.text, identifier)


class TextualNote(TEITag):

    target = 'note'

    def get_replacement(self):
        if self.empty():
            return

        marker = self.get_marker()
        if marker is False or self.get('type') != 'annotation':
            self.raise_()

        abbreviation = self.get('ln')
        if abbreviation:
            return self.abbreviated_lemma(marker, abbreviation)
        return self.unabbreviated_lemma(marker)

    def abbreviated_lemma(self, marker, abbreviation):
        marker.replace_w_str('\\annotationlem{')
        abbreviation = self._process_text_contents(abbreviation)
        return '}{%s}{%s}' % (abbreviation, self.text)

    def unabbreviated_lemma(self, marker):
        marker.replace_w_str('\\annotation{')
        return '}{%s}' % self.text

    def empty(self):
        return self.text is None and len(self.getchildren()) == 0

    def get_marker(self):
        skip = 0
        for n in self.itersiblings('{*}note', preceding=True):
            if n.text:
                skip += 1
            else:
                if skip == 0:
                    return n
                skip = skip - 1
        return False


class Paragraph(TEITag):
    target = 'p'

    def get_replacement(self):

        in_bio = self.getparent().localname == 'trait'

        if in_bio:
            if not self.text:
                return ''
        try:
            self.text = self.text.strip()
        except AttributeError:
            self.raise_()

        paratext = self.text.replace('\n', ' ')

        if in_bio or not self._check_if_in_body():
            if self.getnext() is not None or self.tail:
                return '%s\\par ' % paratext
            else:
                return paratext

        rend = self.get('rend')
        if rend == 'noindent':
            indent = '\\noindent'
        elif rend == 'indent':
            indent = '\\indent'
        elif rend == 'doubleindent':
            indent = '\\doubleindent'
        else:
            # Our headers are wrapped in an eledmac paragraph.
            # So latex doesn't know not to indent them.
            indent = self._after_head_indent()

        return ' '.join(['\n\\pstart', indent, paratext, '\\pend'])

    def _after_head_indent(self):
        previous = self.getprevious()
        if previous is not None and previous.localname == 'head':
            # There was a tag there, but it's been processed already.
            if previous.tail and previous.tail.strip():
                return ''
            # We're the first thing following.
            else:
                return '\\noindent'
        # Not a head. Default to no indent.
        return ''


class VerseLineGroup(TEITag):
    target = 'lg'

    def _init(self):
        super()._init()
        if self.tail:
            self.tail = self.tail.strip()

    def get_replacement(self):
        stanzatext = self.text.strip()
        stanzatext = stanzatext.rstrip('&')
        return '\n\n\\stanza\n%s \\&\n\n' % stanzatext


class VerseLine(TEITag):
    target = 'l'

    def get_replacement(self):
        if self.tail:
            self.tail = self.tail.strip()
        if self.text:
            self.text = self.text.strip()

        return '%s &\n' % self.text


class Foreign(TEITag):
    target = 'foreign'

    def get_replacement(self):
        langcode = self.get('%slang' % XML_NAMESPACE)
        language = LANGUAGES.get(langcode)
        if language:
            return '\\text%s{%s}' % (language, self.text)
        return self.text


class FloatingText(TEITag):
    target = 'floatingText'

    def get_replacement(self):
        floattype = self.get('type')
        if floattype == 'verse':
            return '\n\\vspace{5mm}%s\\vspace{5mm}\n' % self.text
        elif floattype == 'addition':
            return '\\pstart %s\\pend' % self.text
        self.raise_()


class ExternalReferenceTag(TEITag):

    def ex_ref(self, key):
        key = self.required_key(key)
        return key[1:]

    def key_or_empty(self, key):
        return self.get(key) or ''


class Ptr(ExternalReferenceTag, FmtMethods):

    target = 'ptr'

    def get_replacement(self):
        target = self.ex_ref('target')
        kind = self.get('type')

        bib = kind == 'bibliog'
        crossref = kind == 'crossref'

        if bib:
            pre = self.key_or_empty('pre')
            if pre:
                pre = self.square_brackets(pre)
            n = self.key_or_empty('n')
            if pre or n:
                n = self.square_brackets(n)
            return ' \\autocite%s%s{%s}' % (pre, n, target)
        elif crossref:
            return '\\pageref{%s}' % target

        self.raise_()


class PersName(ExternalReferenceTag):
    target = 'persName'

    def _init(self):
        super()._init()
        self.in_body = self._check_if_in_body()

    def get_replacement(self, persdict, in_body):
        ref = self.ex_ref('ref')
        if ref == '??':
            return '\\unknownperson{%s}' % self.text

        if not in_body:
            self.in_body = False

        person = persdict[ref]
        indexonly = person.indexonly is True
        indexname = person.indexname
        if indexonly or not self.in_body:
            if not self.in_body:
                indexname += '|innote'
            return '\\indexperson{%s}{%s}' % (indexname, self.text)
        else:
            description = person.description
            dt = (ref, indexname, description, self.text)
            return '\\person{%s}{%s}{%s}{%s}' % dt


class Label(TEITag):
    target = 'label'

    def get_replacement(self):
        n = self.required_key('n')
        return '\\label{%s}' % n


class Space(TEITag):
    target = 'space'

    def get_replacement(self):
        n = self.get('n')
        if n == 'vertical':
            return '\\bigskip'
        elif n == 'horizontal':
            return '\\hfill{}'
        else:
            return '\\qquad{}'
        self.raise_()


class Div(TEITag):
    target = 'div'

    def get_replacement(self):
        divtype = self.get('type')
        if not divtype:
            return self.unwrap()
        elif divtype in ['month', 'year']:
            date = self.required_key('n')
            if divtype == 'month':
                year = self._find_year()
                assert year
                before = '\n\\addcontentsline{toc}{chapter}\
                      {%s %s}' % (date, year)
            elif divtype == 'year':
                before = '\n\\addcontentsline{toc}{part}{%s}' % date

            self.add_to_previous(before)
        return self.unwrap()

    def _find_year(self):
        for parent in self.iterancestors('{*}div'):
            if parent.get('type') == 'year':
                return parent.required_key('n')


class PageBreak(TEITag):
    target = 'pb'

    def _init(self):
        super()._init()
        has_tail = self.tail is not None
        previous = self.getprevious()
        has_previous = previous is not None

        if has_tail:
            self.tail = self.tail.lstrip()
        if has_previous:
            if previous.tail:
                previous.tail = previous.tail.rstrip()

    def get_replacement(self):

        pagenumber = self.required_key('n')
        pagenumber = pagenumber.lstrip('0')
        if pagenumber == '1':
            return self.delete()

        parent = self.getparent()
        try:
            in_text = parent.localname in ['p', 'lg']
        except AttributeError:
            self.raise_()

        if in_text:
            return ' \\intextpagebreak{[%s]} ' % pagenumber
        else:
            return '\n\\floatpagebreak{[%s]}\n' % pagenumber


class Deletion(TEITag):
    target = 'del'

    def get_replacement(self):
        if self.get('resp'):
            return self.delete()
        elif self.get('hand'):
            return '\\sout{%s}' % self.text
        else:
            self.raise_()


class Add(TEITag):
    target = 'add'

    def get_replacement(self):
        return '\\addition{%s}' % self.text


class FilterTag(TEITag):

    def filterchildren(self, targets):
        searchtargets = map(lambda s: '{*}%s' % s, targets)
        for x in self.iterchildren(searchtargets):
            yield x.localname, x
            targets.remove(x.localname)

        if len(targets) != 0:
            self.raise_()


class Choice(FilterTag):
    target = 'choice'

    def get_replacement(self):
        for name, tag in self.filterchildren(['sic', 'corr']):
            if name == 'sic':
                sic_text = tag.text
            elif name == 'corr':
                corr_text = tag.text

        return '\\correction{%s}{%s}' % (corr_text, sic_text)


class TextualVariant(FilterTag):
    target = 'app'

    def get_replacement(self):
        rdgs = []
        for name, tag in self.filterchildren(['lem', 'rdg']):
            if name == 'lem':
                lem_text = tag.text
            elif name == 'rdg':
                rdgs.append(tag)

        rdgs = map(self.rdgs_map, rdgs)
        return ' \\variants{%s}{%s}' % (lem_text, ' '.join(rdgs))

    @staticmethod
    def rdgs_map(rdg):
        try:
            wit = rdg.attrib['wit']
        except KeyError:
            rdg.raise_()
        return '\\wit{%s}{%s}' % (rdg.text, wit)

# FURTHER SUBCLASSING.


class SoCalled(TEITag, FmtMethods):
    target = 'soCalled'

    def get_replacement(self):
        return self.single_quotes()


class Supplied(TEITag, FmtMethods):
    target = 'supplied'

    def get_replacement(self):
        return self._wrap('«', '»')


class RendTag(TEITag, FmtMethods):

    def _rendery(self, required=False):
        if not required:
            rend = self.get('rend')
        else:
            rend = self.required_key('rend')
        for targets, func in self.fmt_lists():
            if rend in targets:
                return func()
        self.raise_()


class Bibl(RendTag):
    target = 'bibl'

    def get_replacement(self):
        return self._rendery()


class Hi(RendTag):
    target = 'hi'

    def get_replacement(self):
        return self._rendery(required=True)


class Quote(RendTag):
    target = 'q'

    def handle_none(self):
        return self.single_quotes()

    def get_replacement(self):
        return self._rendery()

# SIMPLE ONES:


class DeleteMe(TEITag):

    """Delete tags of this type"""

    target = ['lb']

    def get_replacement(self):
        return self.delete()


class DontTouchMe(TEITag):

    """Take no action with tags of this type"""

    target = ['corr', 'sic', 'lem', 'rdg']

    def get_replacement(self):
        return None


class ReplaceMeWText(TEITag):

    """Replace tags of this type with their text"""

    target = ['body', 'time', 'list', 'item', 'name']

    def get_replacement(self):
        return self.text


class UnWrapMe(TEITag):

    """Unwrap tags of this type"""

    target = ['subst', 'trait']

    def get_replacement(self):
        return self.unwrap()



import yaml
import os

with open(os.path.join(os.path.dirname(__file__), "config.yaml"), "r") as f:
    config = yaml.load(f)


def update_config(curdir):

	custom_settings = os.path.join(curdir, 'resources', 'config.yaml')
	try:
		with open(custom_settings) as f:
			custom_settings = yaml.load(f)
		config.extend(custom_settings)
	except FileNotFoundError:
		pass
